@@ -16,7 +16,7 @@
 #include <sys/types.h>
 #include <sys/ioctl.h>
 #include <sys/timerfd.h>
-#include <sys/poll.h>
+#include <poll.h>
 #include "liburing.h"
 
 #define QD	64
